from typing import FrozenSet

from monkeytypes import InfectionMonkeyBaseModel
from pydantic import Field


class RDPOptions(InfectionMonkeyBaseModel):
    agent_binary_upload_timeout: float = Field(
        default=30.0,
        gt=0.0,
        le=100.0,
        title="Agent Binary Upload Timeout",
        description=(
            "The timeout (in seconds) for uploading the Agent binary to the target machine."
        ),
    )
    domains: FrozenSet[str] = Field(
        default=frozenset(("WORKGROUP",)),
        frozen=True,
        title="Domains",
        description="The Windows domains to try when logging in to the target machine.",
    )
    login_timeout: float = Field(
        default=60.0,
        gt=0.0,
        le=100.0,
        title="Login Timeout",
        description=(
            "The maximum time (in seconds) to wait for the desktop to load after logging in."
        ),
    )
    rdp_connect_timeout: float = Field(
        default=15.0,
        gt=0.0,
        le=100.0,
        title="RDP Connection Timeout",
        description="The maximum time (in seconds) to wait for a response on an RDP connection.",
    )
